@@ -1,6 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -38,7 +37,6 @@
 #include "H5Iprivate.h"  /* IDs                                      */
 #include "H5Lprivate.h"  /* Links                                    */
 #include "H5MMprivate.h" /* Memory management                        */
-#include "H5SLprivate.h" /* Skip lists                               */
 
 /****************/
 /* Local Macros */
@@ -55,23 +53,24 @@ typedef struct {
 
 /* User data for application-style iteration over links in a group */
 typedef struct {
-    hid_t              gid;     /* The group ID for the application callback */
-    H5G_link_iterate_t lnk_op;  /* Application callback */
-    void *             op_data; /* Application's op data */
+    hid_t              gid;      /* The group ID for the application callback */
+    H5O_loc_t         *link_loc; /* The object location for the link */
+    H5G_link_iterate_t lnk_op;   /* Application callback */
+    void              *op_data;  /* Application's op data */
 } H5G_iter_appcall_ud_t;
 
 /* User data for recursive traversal over links from a group */
 typedef struct {
     hid_t           gid;           /* The group ID for the starting group */
-    H5G_loc_t *     curr_loc;      /* Location of starting group */
+    H5G_loc_t      *curr_loc;      /* Location of starting group */
     H5_index_t      idx_type;      /* Index to use */
     H5_iter_order_t order;         /* Iteration order within index */
-    H5SL_t *        visited;       /* Skip list for tracking visited nodes */
-    char *          path;          /* Path name of the link */
+    H5SL_t         *visited;       /* Skip list for tracking visited nodes */
+    char           *path;          /* Path name of the link */
     size_t          curr_path_len; /* Current length of the path in the buffer */
     size_t          path_buf_size; /* Size of path buffer */
-    H5L_iterate_t   op;            /* Application callback */
-    void *          op_data;       /* Application's op data */
+    H5L_iterate2_t  op;            /* Application callback */
+    void           *op_data;       /* Application's op data */
 } H5G_iter_visit_ud_t;
 
 /********************/
@@ -83,6 +82,7 @@ typedef struct {
 /********************/
 
 static herr_t H5G__open_oid(H5G_t *grp);
+static herr_t H5G__visit_cb(const H5O_link_t *lnk, void *_udata);
 
 /*********************/
 /* Package Variables */
@@ -122,7 +122,7 @@ H5G__create_named(const H5G_loc_t *loc, const char *name, hid_t lcpl_id, hid_t g
 {
     H5O_obj_create_t ocrt_info;        /* Information for object creation */
     H5G_obj_create_t gcrt_info;        /* Information for group creation */
-    H5G_t *          ret_value = NULL; /* Return value */
+    H5G_t           *ret_value = NULL; /* Return value */
 
     FUNC_ENTER_PACKAGE
 
@@ -174,9 +174,9 @@ done:
 H5G_t *
 H5G__create(H5F_t *file, H5G_obj_create_t *gcrt_info)
 {
-    H5G_t *  grp       = NULL; /*new group			*/
+    H5G_t   *grp       = NULL; /*new group			*/
     unsigned oloc_init = 0;    /* Flag to indicate that the group object location was created successfully */
-    H5G_t *  ret_value = NULL; /* Return value */
+    H5G_t   *ret_value = NULL; /* Return value */
 
     FUNC_ENTER_PACKAGE
 
@@ -245,13 +245,13 @@ done:
 H5G_t *
 H5G__open_name(const H5G_loc_t *loc, const char *name)
 {
-    H5G_t *    grp = NULL;        /* Group to open */
+    H5G_t     *grp = NULL;        /* Group to open */
     H5G_loc_t  grp_loc;           /* Location used to open group */
     H5G_name_t grp_path;          /* Opened object group hier. path */
     H5O_loc_t  grp_oloc;          /* Opened object object location */
     hbool_t    loc_found = FALSE; /* Location at 'name' found */
     H5O_type_t obj_type;          /* Type of object at location */
-    H5G_t *    ret_value = NULL;  /* Return value */
+    H5G_t     *ret_value = NULL;  /* Return value */
 
     FUNC_ENTER_PACKAGE
 
@@ -308,9 +308,9 @@ done:
 H5G_t *
 H5G_open(const H5G_loc_t *loc)
 {
-    H5G_t *       grp = NULL;       /* Group opened */
+    H5G_t        *grp = NULL;       /* Group opened */
     H5G_shared_t *shared_fo;        /* Shared group object */
-    H5G_t *       ret_value = NULL; /* Return value */
+    H5G_t        *ret_value = NULL; /* Return value */
 
     FUNC_ENTER_NOAPI(NULL)
 
@@ -432,37 +432,6 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__open_oid() */
 
-/*-------------------------------------------------------------------------
- * Function:	H5G__close_cb
- *
- * Purpose:	Closes the specified group.
- *
- * Return:	Non-negative on success/Negative on failure
- *
- * Programmer:	Quincey Koziol
- *		Sunday, February 18, 2018
- *
- *-------------------------------------------------------------------------
- */
-herr_t
-H5G__close_cb(H5G_t *grp)
-{
-    herr_t ret_value = SUCCEED; /* Return value */
-
-    FUNC_ENTER_PACKAGE
-
-    /* Check args */
-    HDassert(grp && grp->shared);
-    HDassert(grp->shared->fo_count > 0);
-
-    /* Call actual group close routine */
-    if (H5G_close(grp) < 0)
-        HGOTO_ERROR(H5E_FILE, H5E_CANTCLOSEOBJ, FAIL, "problem closing group")
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5G__close_cb() */
-
 /*-------------------------------------------------------------------------
  * Function:	H5G_close
  *
@@ -532,7 +501,7 @@ H5G_close(H5G_t *grp)
         else
             /* Free object location (i.e. "unhold" the file if appropriate) */
             if (H5O_loc_free(&(grp->oloc)) < 0)
-            HGOTO_ERROR(H5E_SYM, H5E_CANTRELEASE, FAIL, "problem attempting to free location")
+                HGOTO_ERROR(H5E_SYM, H5E_CANTRELEASE, FAIL, "problem attempting to free location")
 
         /* If this group is a mount point and the mount point is the last open
          * reference to the group (i.e. fo_count == 1 now), then attempt to
@@ -726,7 +695,7 @@ H5G_unmount(H5G_t *grp)
 } /* end H5G_unmount() */
 
 /*-------------------------------------------------------------------------
- * Function:	H5G_iterate_cb
+ * Function:	H5G__iterate_cb
  *
  * Purpose:     Callback function for iterating over links in a group
  *
@@ -739,12 +708,12 @@ H5G_unmount(H5G_t *grp)
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5G_iterate_cb(const H5O_link_t *lnk, void *_udata)
+H5G__iterate_cb(const H5O_link_t *lnk, void *_udata)
 {
     H5G_iter_appcall_ud_t *udata     = (H5G_iter_appcall_ud_t *)_udata; /* User data for callback */
     herr_t                 ret_value = H5_ITER_ERROR;                   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT
+    FUNC_ENTER_STATIC
 
     /* Sanity check */
     HDassert(lnk);
@@ -759,10 +728,10 @@ H5G_iterate_cb(const H5O_link_t *lnk, void *_udata)
 #endif /* H5_NO_DEPRECATED_SYMBOLS */
 
         case H5G_LINK_OP_NEW: {
-            H5L_info_t info; /* Link info */
+            H5L_info2_t info; /* Link info */
 
             /* Retrieve the info for the link */
-            if (H5G_link_to_info(lnk, &info) < 0)
+            if (H5G_link_to_info(udata->link_loc, lnk, &info) < 0)
                 HGOTO_ERROR(H5E_SYM, H5E_CANTGET, H5_ITER_ERROR, "unable to get info for link")
 
             /* Make the application callback */
@@ -775,7 +744,7 @@ H5G_iterate_cb(const H5O_link_t *lnk, void *_udata)
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5G_iterate_cb() */
+} /* end H5G__iterate_cb() */
 
 /*-------------------------------------------------------------------------
  * Function:    H5G_iterate
@@ -790,18 +759,18 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G_iterate(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t skip,
+H5G_iterate(H5G_loc_t *loc, const char *group_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t skip,
             hsize_t *last_lnk, const H5G_link_iterate_t *lnk_op, void *op_data)
 {
-    H5G_loc_t             loc;                   /* Location of parent for group */
     hid_t                 gid = H5I_INVALID_HID; /* ID of group to iterate over */
-    H5G_t *               grp = NULL;            /* Pointer to group data structure to iterate over */
+    H5G_t                *grp = NULL;            /* Pointer to group data structure to iterate over */
     H5G_iter_appcall_ud_t udata;                 /* User data for callback */
     herr_t                ret_value = FAIL;      /* Return value */
 
     FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
+    HDassert(loc);
     HDassert(group_name);
     HDassert(last_lnk);
     HDassert(lnk_op && lnk_op->op_func.op_new);
@@ -809,26 +778,25 @@ H5G_iterate(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_o
     /* Open the group on which to operate.  We also create a group ID which
      * we can pass to the application-defined operator.
      */
-    if (H5G_loc(loc_id, &loc) < 0)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a location")
-    if (NULL == (grp = H5G__open_name(&loc, group_name)))
+    if (NULL == (grp = H5G__open_name(loc, group_name)))
         HGOTO_ERROR(H5E_SYM, H5E_CANTOPENOBJ, FAIL, "unable to open group")
-    if ((gid = H5I_register(H5I_GROUP, grp, TRUE)) < 0)
+    if ((gid = H5VL_wrap_register(H5I_GROUP, grp, TRUE)) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register group")
 
     /* Set up user data for callback */
-    udata.gid     = gid;
-    udata.lnk_op  = *lnk_op;
-    udata.op_data = op_data;
+    udata.gid      = gid;
+    udata.link_loc = &grp->oloc;
+    udata.lnk_op   = *lnk_op;
+    udata.op_data  = op_data;
 
     /* Call the real group iteration routine */
     if ((ret_value =
-             H5G__obj_iterate(&(grp->oloc), idx_type, order, skip, last_lnk, H5G_iterate_cb, &udata)) < 0)
+             H5G__obj_iterate(&(grp->oloc), idx_type, order, skip, last_lnk, H5G__iterate_cb, &udata)) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "error iterating over links")
 
 done:
     /* Release the group opened */
-    if (gid > 0) {
+    if (gid != H5I_INVALID_HID) {
         if (H5I_dec_app_ref(gid) < 0)
             HDONE_ERROR(H5E_SYM, H5E_CANTRELEASE, FAIL, "unable to close group")
     }
@@ -839,7 +807,7 @@ done:
 } /* end H5G_iterate() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5G_free_visit_visited
+ * Function:    H5G__free_visit_visited
  *
  * Purpose:     Free the key for an object visited during a group traversal
  *
@@ -851,17 +819,17 @@ done:
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5G_free_visit_visited(void *item, void H5_ATTR_UNUSED *key, void H5_ATTR_UNUSED *operator_data /*in,out*/)
+H5G__free_visit_visited(void *item, void H5_ATTR_UNUSED *key, void H5_ATTR_UNUSED *operator_data /*in,out*/)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    FUNC_ENTER_STATIC_NOERR
 
     item = H5FL_FREE(H5_obj_t, item);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5G_free_visit_visited() */
+} /* end H5G__free_visit_visited() */
 
 /*-------------------------------------------------------------------------
- * Function:	H5G_visit_cb
+ * Function:	H5G__visit_cb
  *
  * Purpose:     Callback function for recursively visiting links from a group
  *
@@ -874,10 +842,10 @@ H5G_free_visit_visited(void *item, void H5_ATTR_UNUSED *key, void H5_ATTR_UNUSED
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5G_visit_cb(const H5O_link_t *lnk, void *_udata)
+H5G__visit_cb(const H5O_link_t *lnk, void *_udata)
 {
     H5G_iter_visit_ud_t *udata = (H5G_iter_visit_ud_t *)_udata; /* User data for callback */
-    H5L_info_t           info;                                  /* Link info */
+    H5L_info2_t          info;                                  /* Link info */
     H5G_loc_t            obj_loc;                               /* Location of object */
     H5G_name_t           obj_path;                              /* Object's group hier. path */
     H5O_loc_t            obj_oloc;                              /* Object's object location */
@@ -887,7 +855,7 @@ H5G_visit_cb(const H5O_link_t *lnk, void *_udata)
     size_t len_needed;                          /* Length of path string needed */
     herr_t ret_value = H5_ITER_CONT;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT
+    FUNC_ENTER_STATIC
 
     /* Sanity check */
     HDassert(lnk);
@@ -913,7 +881,7 @@ H5G_visit_cb(const H5O_link_t *lnk, void *_udata)
     udata->curr_path_len += link_name_len;
 
     /* Construct the link info from the link message */
-    if (H5G_link_to_info(lnk, &info) < 0)
+    if (H5G_link_to_info(udata->curr_loc->oloc, lnk, &info) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTGET, H5_ITER_ERROR, "unable to get info for link")
 
     /* Make the application callback */
@@ -941,15 +909,13 @@ H5G_visit_cb(const H5O_link_t *lnk, void *_udata)
         /* Check if we've seen the object the link references before */
         if (NULL == H5SL_search(udata->visited, &obj_pos)) {
             H5O_type_t otype; /* Basic object type (group, dataset, etc.) */
-            unsigned   rc;    /* Reference count of object    */
 
             /* Get the object's reference count and type */
-            if (H5O_get_rc_and_type(&obj_oloc, &rc, &otype) < 0)
+            if (H5O_get_rc_and_type(&obj_oloc, NULL, &otype) < 0)
                 HGOTO_ERROR(H5E_SYM, H5E_CANTGET, H5_ITER_ERROR, "unable to get object info")
 
-            /* If its ref count is > 1, we add it to the list of visited objects */
-            /* (because it could come up again during traversal) */
-            if (rc > 1) {
+            /* Add it to the list of visited objects */
+            {
                 H5_obj_t *new_node; /* New object node for visited list */
 
                 /* Allocate new object "position" node */
@@ -963,11 +929,11 @@ H5G_visit_cb(const H5O_link_t *lnk, void *_udata)
                 if (H5SL_insert(udata->visited, new_node, new_node) < 0)
                     HGOTO_ERROR(H5E_SYM, H5E_CANTINSERT, H5_ITER_ERROR,
                                 "can't insert object node into visited list")
-            } /* end if */
+            }
 
             /* If it's a group, we recurse into it */
             if (otype == H5O_TYPE_GROUP) {
-                H5G_loc_t * old_loc  = udata->curr_loc; /* Pointer to previous group location info */
+                H5G_loc_t  *old_loc  = udata->curr_loc; /* Pointer to previous group location info */
                 H5_index_t  idx_type = udata->idx_type; /* Type of index to use */
                 H5O_linfo_t linfo;                      /* Link info message */
                 htri_t      linfo_exists;               /* Whether the link info message exists */
@@ -1003,7 +969,7 @@ H5G_visit_cb(const H5O_link_t *lnk, void *_udata)
 
                 /* Iterate over links in group */
                 ret_value = H5G__obj_iterate(&obj_oloc, idx_type, udata->order, (hsize_t)0, NULL,
-                                             H5G_visit_cb, udata);
+                                             H5G__visit_cb, udata);
 
                 /* Restore location */
                 udata->curr_loc = old_loc;
@@ -1021,7 +987,7 @@ done:
         HDONE_ERROR(H5E_SYM, H5E_CANTRELEASE, H5_ITER_ERROR, "can't free location")
 
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5G_visit_cb() */
+} /* end H5G__visit_cb() */
 
 /*-------------------------------------------------------------------------
  * Function:    H5G_visit
@@ -1049,17 +1015,15 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G_visit(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_order_t order, H5L_iterate_t op,
-          void *op_data)
+H5G_visit(H5G_loc_t *loc, const char *group_name, H5_index_t idx_type, H5_iter_order_t order,
+          H5L_iterate2_t op, void *op_data)
 {
     H5G_iter_visit_ud_t udata;                 /* User data for callback */
     H5O_linfo_t         linfo;                 /* Link info message */
     htri_t              linfo_exists;          /* Whether the link info message exists */
     hid_t               gid = H5I_INVALID_HID; /* Group ID */
-    H5G_t *             grp = NULL;            /* Group opened */
-    H5G_loc_t           loc;                   /* Location of group passed in */
+    H5G_t              *grp = NULL;            /* Group opened */
     H5G_loc_t           start_loc;             /* Location of starting group */
-    unsigned            rc;                    /* Reference count of object    */
     herr_t              ret_value = FAIL;      /* Return value */
 
     /* Portably clear udata struct (before FUNC_ENTER) */
@@ -1068,15 +1032,15 @@ H5G_visit(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_ord
     FUNC_ENTER_NOAPI(FAIL)
 
     /* Check args */
-    if (H5G_loc(loc_id, &loc) < 0)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a location")
+    if (!loc)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "loc parameter cannot be NULL")
 
     /* Open the group to begin visiting within */
-    if (NULL == (grp = H5G__open_name(&loc, group_name)))
+    if (NULL == (grp = H5G__open_name(loc, group_name)))
         HGOTO_ERROR(H5E_SYM, H5E_CANTOPENOBJ, FAIL, "unable to open group")
 
     /* Register an ID for the starting group */
-    if ((gid = H5I_register(H5I_GROUP, grp, TRUE)) < 0)
+    if ((gid = H5VL_wrap_register(H5I_GROUP, grp, TRUE)) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register group")
 
     /* Get the location of the starting group */
@@ -1101,13 +1065,8 @@ H5G_visit(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_ord
     if ((udata.visited = H5SL_create(H5SL_TYPE_OBJ, NULL)) == NULL)
         HGOTO_ERROR(H5E_SYM, H5E_CANTCREATE, FAIL, "can't create skip list for visited objects")
 
-    /* Get the group's reference count */
-    if (H5O_get_rc_and_type(&grp->oloc, &rc, NULL) < 0)
-        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to get object info")
-
-    /* If its ref count is > 1, we add it to the list of visited objects */
-    /* (because it could come up again during traversal) */
-    if (rc > 1) {
+    /* Add it to the list of visited objects */
+    {
         H5_obj_t *obj_pos; /* New object node for visited list */
 
         /* Allocate new object "position" node */
@@ -1121,7 +1080,7 @@ H5G_visit(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_ord
         /* Add to list of visited objects */
         if (H5SL_insert(udata.visited, obj_pos, obj_pos) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINSERT, FAIL, "can't insert object node into visited list")
-    } /* end if */
+    }
 
     /* Attempt to get the link info for this group */
     if ((linfo_exists = H5G__obj_get_linfo(&(grp->oloc), &linfo)) < 0)
@@ -1146,17 +1105,17 @@ H5G_visit(hid_t loc_id, const char *group_name, H5_index_t idx_type, H5_iter_ord
 
     /* Call the link iteration routine */
     if ((ret_value =
-             H5G__obj_iterate(&(grp->oloc), idx_type, order, (hsize_t)0, NULL, H5G_visit_cb, &udata)) < 0)
+             H5G__obj_iterate(&(grp->oloc), idx_type, order, (hsize_t)0, NULL, H5G__visit_cb, &udata)) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "can't visit links")
 
 done:
     /* Release user data resources */
     H5MM_xfree(udata.path);
     if (udata.visited)
-        H5SL_destroy(udata.visited, H5G_free_visit_visited, NULL);
+        H5SL_destroy(udata.visited, H5G__free_visit_visited, NULL);
 
     /* Release the group opened */
-    if (gid > 0) {
+    if (gid != H5I_INVALID_HID) {
         if (H5I_dec_app_ref(gid) < 0)
             HDONE_ERROR(H5E_SYM, H5E_CANTRELEASE, FAIL, "unable to close group")
     }
